@@ -9,6 +9,7 @@ import json
 import socket
 import asyncio
 import threading
+import ssl
 
 # TODO: $iothub/methods/POST/# is unnecessary in topics
 # TODO: on_settings does not work
@@ -70,7 +71,7 @@ class Module:
         return f"SharedAccessSignature sr={resource}&sig={signature}&se={expiry}"
 
     def get_connect_options(self):
-        host = os.environ['IOTEDGE_GATEWAYHOSTNAME']
+        self.host = host = os.environ['IOTEDGE_GATEWAYHOSTNAME']
         self.device_id = os.environ['IOTEDGE_DEVICEID']
         self.module_id = os.environ['IOTEDGE_MODULEID']
         self.iot_hub_hostname = os.environ['IOTEDGE_IOTHUBHOSTNAME']
@@ -252,3 +253,26 @@ class Module:
         self.client.subscribe(topic)
         self.topics.append(topic)
         self.client.message_callback_add(topic, handler)
+
+    async def invoke(self, moduleId: str, method: str, payload={}, connect_timeout=15, response_timeout=15):
+        myssl = ssl.create_default_context()
+        myssl.check_hostname = False
+        myssl.verify_mode = ssl.CERT_NONE
+
+        data = {
+            "methodName": method,
+            "payload": payload,
+            "connectTimeoutInSeconds": connect_timeout,
+            "responseTimeoutInSeconds": response_timeout
+        }
+
+        req = urllib.request.Request(
+            f"https://{self.host}/twins/{urllib.parse.quote(self.device_id)}/modules/{urllib.parse.quote(moduleId)}/methods")
+        req.add_header('content-type', 'application/json')
+        req.add_header('x-ms-edge-moduleId',
+                       f"{self.device_id}/{self.module_id}")
+        req.add_header('authorization', self.get_module_token())
+        req.data = json.dumps(data).encode("utf-8")
+
+        contents: bytes = urllib.request.urlopen(req, context=myssl).read()
+        return json.loads(contents.decode('utf-8'))
